@@ -99,6 +99,11 @@ class TestGatewayWebserver:
         assert response.status_code == 404
         assert response.json() == {"detail": "Not Found"}
 
+    def test_static_cachecontrol(self, rest_client: TestClient):
+        response = rest_client.get("/static/favicon.png")
+        assert response.status_code == 200
+        assert response.headers["Cache-Control"] == "public, max-age=604800"
+
     ######################
     # Core Functionality #
     def test_log_viewer(self, rest_client: TestClient):
@@ -434,6 +439,9 @@ class TestGatewayWebserver:
         gateway_client = GatewayClient(GatewayClientConfig(port=gateway.settings.PORT, authenticate=True, api_key="test"))
         self._wait_for_data(rest_client=rest_client)
         response_state = gateway_client.state()
+        assert sorted(list(gateway_client._openapi_spec.keys())) == ["components", "info", "openapi", "paths"]
+        assert "/api/v1/last/example" in gateway_client._openapi_spec["paths"].keys()
+        assert "/api/v1/last/example_list" in gateway_client._openapi_spec["paths"].keys()
         assert response_state == ["example"]
 
         for route in ["example", "example_list"]:
